@@ -43,14 +43,14 @@ gr_make_skiphead (size_t itemsize, uint64_t nitems_to_skip)
 
 int
 gr_skiphead::general_work(int noutput_items,
-			  gr_vector_int &ninput_items_ignored,
+			  gr_vector_int &ninput_items_,
 			  gr_vector_const_void_star &input_items,
 			  gr_vector_void_star &output_items)
 {
   const char *in = (const char *) input_items[0];
   char *out = (char *) output_items[0];
 
-  int ninput_items = noutput_items;	// we've got at least this many input items
+  int ninput_items = std::min(ninput_items_[0], noutput_items);
   int ii = 0;				// input index
 
   while (ii < ninput_items){
