@@ -31,8 +31,9 @@
 #include <volk/volk.h>
 #include <iostream>
 #include <map>
+#include <boost/format.hpp>
 
-#define GR_FLAT_FLOWGRAPH_DEBUG 0
+#define GR_FLAT_FLOWGRAPH_DEBUG  0
 
 // 32Kbyte buffer size between blocks
 #define GR_FIXED_BUFFER_SIZE (32*(1L<<10))
@@ -71,6 +72,15 @@ gr_flat_flowgraph::setup_connections()
     block->set_is_unaligned(false);
   }
 
+  // Connect message ports connetions
+  for(gr_msg_edge_viter_t i = d_msg_edges.begin(); i != d_msg_edges.end(); i++){
+    if(GR_FLAT_FLOWGRAPH_DEBUG)
+        std::cout << boost::format("flat_fg connecting msg primitives: (%s, %s)->(%s, %s)\n") %
+                    i->src().block() % i->src().port() %
+                    i->dst().block() % i->dst().port();
+    i->src().block()->message_port_sub( i->src().port(), pmt::pmt_cons(i->dst().block()->alias_pmt(), i->dst().port()) );
+    }
+
 }
 
 gr_block_detail_sptr
@@ -80,14 +90,23 @@ gr_flat_flowgraph::allocate_block_detail(gr_basic_block_sptr block)
   int noutputs = calc_used_ports(block, false).size();
   gr_block_detail_sptr detail = gr_make_block_detail(ninputs, noutputs);
 
+  gr_block_sptr grblock = cast_to_block_sptr(block);
+  if(!grblock)
+    throw std::runtime_error("allocate_block_detail found non-gr_block");
+
   if (GR_FLAT_FLOWGRAPH_DEBUG)
     std::cout << "Creating block detail for " << block << std::endl;
 
   for (int i = 0; i < noutputs; i++) {
+    grblock->expand_minmax_buffer(i);
+
     gr_buffer_sptr buffer = allocate_buffer(block, i);
     if (GR_FLAT_FLOWGRAPH_DEBUG)
       std::cout << "Allocated buffer for output " << block << ":" << i << std::endl;
     detail->set_output(i, buffer);
+
+    // Update the block's max_output_buffer based on what was actually allocated.
+    grblock->set_max_output_buffer(i, buffer->bufsize());
   }
 
   return detail;
@@ -114,6 +133,21 @@ gr_flat_flowgraph::allocate_buffer(gr_basic_block_sptr block, int port)
   // ensure we have a buffer at least twice their decimation factor*output_multiple
   gr_basic_block_vector_t blocks = calc_downstream_blocks(block, port);
 
+  // limit buffer size if indicated
+  if(grblock->max_output_buffer(port) > 0) {
+//    std::cout << "constraining output items to " << block->max_output_buffer(port) << "\n";
+    nitems = std::min((long)nitems, (long)grblock->max_output_buffer(port));
+    nitems -= nitems%grblock->output_multiple();
+    if( nitems < 1 )
+      throw std::runtime_error("problems allocating a buffer with the given max output buffer constraint!");
+  }
+  else if(grblock->min_output_buffer(port) > 0) {
+    nitems = std::max((long)nitems, (long)grblock->min_output_buffer(port));
+    nitems -= nitems%grblock->output_multiple();
+    if( nitems < 1 )
+      throw std::runtime_error("problems allocating a buffer with the given min output buffer constraint!");
+  }
+
   for (gr_basic_block_viter_t p = blocks.begin(); p != blocks.end(); p++) {
     gr_block_sptr dgrblock = cast_to_block_sptr(*p);
     if (!dgrblock)
@@ -125,6 +159,7 @@ gr_flat_flowgraph::allocate_buffer(gr_basic_block_sptr block, int port)
     nitems = std::max(nitems, static_cast<int>(2*(decimation*multiple+history)));
   }
 
+//  std::cout << "gr_make_buffer(" << nitems << ", " << item_size << ", " << grblock << "\n";
   return gr_make_buffer(nitems, item_size, grblock);
 }
 
@@ -267,10 +302,10 @@ gr_flat_flowgraph::setup_buffer_alignment(gr_block_sptr block)
   for(int i = 0; i < block->detail()->ninputs(); i++) {
     void *r = (void*)block->detail()->input(i)->read_pointer();
     unsigned long int ri = (unsigned long int)r % alignment;
-    //std::cout << "reader: " << r << "  alignment: " << ri << std::endl;
+    //std::cerr << "reader: " << r << "  alignment: " << ri << std::endl;
     if(ri != 0) {
       size_t itemsize = block->detail()->input(i)->get_sizeof_item();
-      block->detail()->input(i)->update_read_pointer(ri/itemsize);
+      block->detail()->input(i)->update_read_pointer(alignment-ri/itemsize);
     }
     block->set_unaligned(0);
     block->set_is_unaligned(false);
@@ -279,10 +314,10 @@ gr_flat_flowgraph::setup_buffer_alignment(gr_block_sptr block)
   for(int i = 0; i < block->detail()->noutputs(); i++) {
     void *w = (void*)block->detail()->output(i)->write_pointer();
     unsigned long int wi = (unsigned long int)w % alignment;
-    size_t itemsize = block->detail()->output(i)->get_sizeof_item();
-    //std::cout << "writer: " << w << "  alignment: " << wi << std::endl;
+    //std::cerr << "writer: " << w << "  alignment: " << wi << std::endl;
     if(wi != 0) {
-      block->detail()->output(i)->update_write_pointer(wi/itemsize);
+      size_t itemsize = block->detail()->output(i)->get_sizeof_item();
+      block->detail()->output(i)->update_write_pointer(alignment-wi/itemsize);
     }
     block->set_unaligned(0);
     block->set_is_unaligned(false);
@@ -325,3 +360,44 @@ gr_flat_flowgraph::make_block_vector(gr_basic_block_vector_t &blocks)
 
   return result;
 }
+
+
+void gr_flat_flowgraph::clear_endpoint(const gr_msg_endpoint &e, bool is_src){
+    for(size_t i=0; i<d_msg_edges.size(); i++){
+        if(is_src){
+            if(d_msg_edges[i].src() == e){
+                d_msg_edges.erase(d_msg_edges.begin() + i);
+                i--;
+            }
+        } else {
+            if(d_msg_edges[i].dst() == e){
+                d_msg_edges.erase(d_msg_edges.begin() + i);
+                i--;
+            }
+        }
+    }
+}
+
+void gr_flat_flowgraph::replace_endpoint(const gr_msg_endpoint &e, const gr_msg_endpoint &r, bool is_src){
+    size_t n_replr(0);
+    if(GR_FLAT_FLOWGRAPH_DEBUG)
+        std::cout << boost::format("gr_flat_flowgraph::replace_endpoint( %s, %s, %d )\n") % e.block()% r.block()% is_src;
+    for(size_t i=0; i<d_msg_edges.size(); i++){
+        if(is_src){
+            if(d_msg_edges[i].src() == e){
+                if(GR_FLAT_FLOWGRAPH_DEBUG)
+                    std::cout << boost::format("gr_flat_flowgraph::replace_endpoint() flattening to ( %s, %s )\n") % r.block()% d_msg_edges[i].dst().block();
+                d_msg_edges.push_back( gr_msg_edge(r, d_msg_edges[i].dst() ) );
+                n_replr++;
+            }
+        } else {
+            if(d_msg_edges[i].dst() == e){
+                if(GR_FLAT_FLOWGRAPH_DEBUG)
+                    std::cout << boost::format("gr_flat_flowgraph::replace_endpoint() flattening to ( %s, %s )\n") % r.block()% d_msg_edges[i].dst().block();
+                d_msg_edges.push_back( gr_msg_edge(d_msg_edges[i].src(), r ) );
+                n_replr++;
+            }
+        }
+    }
+}
+
