@@ -23,6 +23,7 @@
 #ifndef _GR_VMCIRCBUF_CREATEFILEMAPPING_H_
 #define _GR_VMCIRCBUF_CREATEFILEMAPPING_H_
 
+#include <gr_core_api.h>
 #include <gr_vmcircbuf.h>
 
 #ifdef HAVE_CREATEFILEMAPPING
@@ -32,7 +33,7 @@
  * \brief concrete class to implement circular buffers with mmap and shm_open
  * \ingroup internal
  */
-class gr_vmcircbuf_createfilemapping : public gr_vmcircbuf
+class GR_CORE_API gr_vmcircbuf_createfilemapping : public gr_vmcircbuf
 {
  public:
    // CREATORS
@@ -49,7 +50,7 @@ class gr_vmcircbuf_createfilemapping : public gr_vmcircbuf
 /*!
  * \brief concrete factory for circular buffers built using mmap and shm_open
  */
-class gr_vmcircbuf_createfilemapping_factory : public gr_vmcircbuf_factory
+class GR_CORE_API gr_vmcircbuf_createfilemapping_factory : public gr_vmcircbuf_factory
 {
  private:
   static gr_vmcircbuf_factory *s_the_factory;
