@@ -65,15 +65,11 @@ void qa_set_msg_handler::t0()
   tb->start();
 
   // Send them...
+  pmt_t port(pmt_intern("port"));
   for (int i = 0; i < NMSGS; i++){
-    send(nop, mp(mp("example-msg"), mp(i)));
+    send(nop, port, mp(mp("example-msg"), mp(i)));
   }
 
-  // And send a message to null_source to confirm that the default
-  // message handling action (which should be a nop) doesn't dump
-  // core.
-  send(src, mp(mp("example-msg"), mp(0)));
-
   // Give the messages a chance to be processed
   boost::this_thread::sleep(boost::posix_time::milliseconds(100));
 
