@@ -1,5 +1,5 @@
 #
-# Copyright 2003,2004,2006,2008,2009,2010 Free Software Foundation, Inc.
+# Copyright 2003-2012 Free Software Foundation, Inc.
 #
 # This file is part of GNU Radio
 #
@@ -24,36 +24,19 @@
 # This is the main GNU Radio python module.
 # We pull the swig output and the other modules into the gnuradio.gr namespace
 
-# Temporary workaround for ticket:181.
-# Use leading underscores to avoid namespace pollution
-import sys
-_RTLD_GLOBAL = 0
-try:
-    from dl import RTLD_GLOBAL as _RTLD_GLOBAL
-except ImportError:
-    try:
-	from DLFCN import RTLD_GLOBAL as _RTLD_GLOBAL
-    except ImportError:
-	pass
-
-if _RTLD_GLOBAL != 0:
-    _dlopenflags = sys.getdlopenflags()
-    sys.setdlopenflags(_dlopenflags|_RTLD_GLOBAL)
-
 from gnuradio_core import *
 from exceptions import *
 from hier_block2 import *
 from top_block import *
-
-if _RTLD_GLOBAL != 0:
-    sys.setdlopenflags(_dlopenflags)             # Restore original flags
+from gateway import basic_block, sync_block, decim_block, interp_block
+from tag_utils import tag_to_python, tag_to_pmt
 
 # create a couple of aliases
 serial_to_parallel = stream_to_vector
 parallel_to_serial = vector_to_stream
 
 # Force the preference database to be initialized
-from prefs import prefs
+prefs = gr_prefs.singleton
 
 #alias old gr_add_vXX and gr_multiply_vXX
 add_vcc = add_cc
