@@ -24,10 +24,13 @@
 #include <stdexcept>
 #include <iostream>
 #include <boost/format.hpp>
+#include <boost/thread/thread.hpp>
 #include <boost/make_shared.hpp>
 #include "gr_uhd_common.h"
 
 static const pmt::pmt_t TIME_KEY = pmt::pmt_string_to_symbol("rx_time");
+static const pmt::pmt_t RATE_KEY = pmt::pmt_string_to_symbol("rx_rate");
+static const pmt::pmt_t FREQ_KEY = pmt::pmt_string_to_symbol("rx_freq");
 
 #include <uhd/convert.hpp>
 inline gr_io_signature_sptr args_to_io_sig(const uhd::stream_args_t &args){
@@ -88,6 +91,8 @@ public:
 
     void set_samp_rate(double rate){
         _dev->set_rx_rate(rate);
+        _samp_rate = this->get_samp_rate();
+        _tag_now = true;
     }
 
     double get_samp_rate(void){
@@ -105,7 +110,10 @@ public:
     uhd::tune_result_t set_center_freq(
         const uhd::tune_request_t tune_request, size_t chan
     ){
-        return _dev->set_rx_freq(tune_request, chan);
+        const uhd::tune_result_t res = _dev->set_rx_freq(tune_request, chan);
+        _center_freq = this->get_center_freq(chan);
+        _tag_now = true;
+        return res;
     }
 
     double get_center_freq(size_t chan){
@@ -330,9 +338,10 @@ public:
 
         //If receive resulted in a timeout condition:
         //We now receive a single packet with a large timeout.
-        if (_metadata.error_code == uhd::rx_metadata_t::ERROR_CODE_TIMEOUT){
+        if (_metadata.error_code == uhd::rx_metadata_t::ERROR_CODE_TIMEOUT)
+        {
             num_samps = _rx_stream->recv(
-                output_items, noutput_items, _metadata, 1.0, true/*one pkt*/
+                output_items, noutput_items, _metadata, 0.1, true/*one pkt*/
             );
         }
         #else
@@ -358,17 +367,18 @@ public:
                     pmt::pmt_from_uint64(_metadata.time_spec.get_full_secs()),
                     pmt::pmt_from_double(_metadata.time_spec.get_frac_secs())
                 );
-                //create a timestamp tag for each channel
+                //create a tag set for each channel
                 for (size_t i = 0; i < _nchan; i++){
                     this->add_item_tag(i, nitems_written(0), TIME_KEY, val, _id);
+                    this->add_item_tag(i, nitems_written(0), RATE_KEY, pmt::pmt_from_double(_samp_rate), _id);
+                    this->add_item_tag(i, nitems_written(0), FREQ_KEY, pmt::pmt_from_double(_center_freq), _id);
                 }
             }
             break;
 
         case uhd::rx_metadata_t::ERROR_CODE_TIMEOUT:
-            //Assume that the user called stop() on the flow graph.
-            //However, a timeout can occur under error conditions.
-            return WORK_DONE;
+            //its ok to timeout, perhaps the user is doing finite streaming
+            return 0;
 
         case uhd::rx_metadata_t::ERROR_CODE_OVERFLOW:
             _tag_now = true;
@@ -391,6 +401,11 @@ public:
         _stream_now = false;
     }
 
+    void issue_stream_cmd(const uhd::stream_cmd_t &cmd)
+    {
+        _dev->issue_stream_cmd(cmd);
+    }
+
     bool start(void){
         #ifdef GR_UHD_USE_STREAM_API
         _rx_stream = _dev->get_rx_stream(_stream_args);
@@ -508,6 +523,10 @@ private:
 
     uhd::time_spec_t _start_time;
     bool _start_time_set;
+
+    //tag shadows
+    double _samp_rate;
+    double _center_freq;
 };
 
 
