@@ -49,6 +49,9 @@ TOOLBAR_LIST = (
 	None,
 	Actions.BLOCK_ENABLE,
 	Actions.BLOCK_DISABLE,
+    None,
+    Actions.RELOAD_BLOCKS,
+    Actions.OPEN_HIER,
 )
 
 ##The list of actions and categories for the menu bar.
