@@ -116,7 +116,7 @@ class Port(_Port, _GUIPort):
 		_Port.validate(self)
 		if not self.get_enabled_connections() and not self.get_optional():
 			self.add_error_message('Port is not connected.')
-		if not self.is_source() and len(self.get_enabled_connections()) > 1:
+		if not self.is_source() and (not self.get_type() == "message") and len(self.get_enabled_connections()) > 1:
 			self.add_error_message('Port has too many connections.')
 		#message port logic
 		if self.get_type() == 'msg':
